parseData();
function parseData() {
  const fs = require('fs');
  // Dictionaries used for the info conversion
  const context_dict = {1:"3P", 2:"1P", 3:"VMU-MC"};
  const dim_dict = {"INT32": 4, "INT64": 8, "UINT32": 4, "UINT64": 8, "INT16": 2, "UINT16": 2};
  
  // INPUT SECTION
  // Example of input data; actually the script is built to print all to the terminal, recursively for each element into the array (dual-dimensional)
  const packet_list = [
    [1, 107, 132, 30, 233, 0, 115, 0, 89, 0, 1, 123, 64, 146, 210, 0, 99, 248, 19, 233, 0, 60, 93, 34, 35, 0, 0, 0, 0, 0, 255, 119, 8, 12, 13, 46, 43],
    [0x01, 0x6B, 0xD8, 0x5B, 0xE9, 0x00, 0x73, 0x00, 0x59, 0x00, 0x01, 0x7B, 0x40, 0x92, 0xD2, 0x00, 0x63, 0xF8, 0x13, 0xE9, 0x00, 0x93, 0xF8, 0xA0, 0x93, 0x00, 0x9B, 0x70, 0xD8, 0x93, 0x00, 0x3C, 0x52, 0x0B, 0x23, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFF, 0x77, 0x08, 0x0B, 0x12, 0x0F, 0x00]
  ];
  
  // Path to the file with the properties
  const header_path = "loraHeaders.json";
  
  // Collecting all the properties
  const data = fs.readFileSync(header_path, 'utf8');
  const headers_json = JSON.parse(data);
  
  for (const packet of packet_list) {
    // Decrypting engine
    const context = packet[0];
    const dim = packet.length;
    const context_headers = headers_json[context_dict[context]];
    let counter = 0;
  
    while (true) {
      counter += 1;
      const header = packet[counter];
      if (header !== 255) {
        const single_header_container = context_headers.find(item => parseInt(item.header) === header);
        if (!single_header_container) {
          throw new Error(`${header} not found in context: ${context_dict[context]}`);
        }
  
        const value_coded = [];
        const var_dimension = dim_dict[single_header_container.format];
        for (let i = 0; i < var_dimension; i++) {
          value_coded.push(packet[counter + i + 1]);
        }
        counter += var_dimension;
  
        let value_decoded = 0;

        for (let i = 0; i < value_coded.length; i++) {
          if (single_header_container.format === "INT16") {
            value_decoded += ([value_coded[i] << (8 * i)])[0];
          } else if (single_header_container.format === "UINT16") {
            value_decoded += ([value_coded[i] << (8 * i)])[0];
          } else if (single_header_container.format === "INT32") {
            value_decoded += ([value_coded[i] << (8 * i)])[0];
          } else if (single_header_container.format === "UINT32") {
            value_decoded += ([value_coded[i] << (8 * i)])[0];
          } else if (single_header_container.format === "INT64") {
            value_decoded += ([value_coded[i] << (8 * i)])[0];
          } else if (single_header_container.format === "UINT64") {
            value_decoded +=  ([value_coded[i] << (8 * i)])[0];
          } else {
            throw new Error(`${single_header_container.format} not managed, header ${header}, ${context_dict[context]}`);
          }
        }
  
        // OUTPUT: Clear printout of the data; here it is possible to change the printout destination
        console.log(`${(value_decoded * parseFloat(single_header_container.uwpMultiplier)).toFixed(3)}    ${single_header_container.variableKey},     ${single_header_container.samplingType}     Header: ${header}`);
      } else {
        // OUTPUT: Clear printout of the timestamp; here it is possible to change the printout destination
        console.log(`UTC break down time: ${packet[counter + 1] + 1900}-${packet[counter + 2] + 1}-${packet[counter + 3]} h:${packet[counter + 4]}-${packet[counter + 5]}-${packet[counter + 6]}\n`);
        counter += 6;
      }
  
      if (counter + 1 >= dim) {
        if (!(counter + 1 === dim)) {
          console.log("The packet is not as long as expected\n");
        }
        break;
      }
    }
  }
}
